#
# Tcl part for dqevh module for smart and easy vhosting
#

#
# custom host->path functions can be written with
# proc dqevh::m.<methodname> {host rootdir param}
#

namespace eval dqevh {}

proc dqevh::do_sre {str} {
    return [string map [list ** .* .. . . \\.] $str]
}
set dqevh::vhostsfile /usr/local/aolserver/vhosts
# readfile {{{
proc dqevh::readfile {} {
    variable vhostsfile
    set fh [open $vhostsfile r]
    set rc [read $fh]
    close $fh
    return $rc
}
# }}}

# main {{{
proc dqevh::main {host rootdir vhosts} {
    global errorInfo
    # cut down the :port stuff - hihi
    regsub ":.*\$" $host "" host
    regsub "^www\." $host "" host
    set done 0
    foreach {me param} $vhosts {
        set cmd [namespace current]::m.$me
        if {[info commands $cmd]!=""} {
            if {[catch [list $cmd $host $rootdir $param] res]} {
                ns_log bug "Error in $cmd:\n$errorInfo"
            }  else  {
                if {[lindex $res 0]} {
                    set rootdir [lindex $res 2]
                }
                if {[lindex $res 1]} {
                    set done 1
                }
            }
        }
        if {$done} {
            break
        }
    }
    ns_log debug "dqevh/Tcl - \"$host\"->\"$rootdir\""
    return $rootdir
}
# }}}
# m.re {{{
proc dqevh::m.re {host rootdir param} {
    set re "^[lindex $param 0]\$"
    set ok 0; set done 0
    if {[regexp $re $host]>0} {
        set ok 1
        set done 1
        regsub $re $host [lindex $param 1] rootdir
    }
    return [list $ok $done $rootdir]
}
# }}}
# m.revpath {{{
proc dqevh::m.revpath {host rootdir param} {
    set suffix [do_sre [lindex $param 0]]
    set prefix ""
    regexp $suffix $host {} prefix
    if {$prefix!=""} {
        set path [list [lindex $param 1]]
        foreach ss [split $prefix .] {
            if {$ss!=""} {
                set path [linsert $path 1 $ss]
            }
        }
        return [list 1 1 [eval file join $path]]
    }  else  {
        return {0 0}
    }
}
proc dqevh::m.sre {host rootdir param} {
    set param [lreplace $param 0 0 [do_sre [lindex $param 0]]]
    return [m.re $host $rootdir $param]
}
# }}}
# m.xuser {{{
proc dqevh::m.xuser {host rootdir param} {
    set suffix [do_sre [lindex $param 0]]
    set prefix ""
    regexp $suffix $host {} prefix
    if {$prefix!=""} {
        #set pre [list]
        #foreach ppre [split $prefix .] {
            #if {$ppre!=""} {lappend pre $ppre}
        #}
        set pre [split $prefix .]
        set user [lindex $pre end]
        set pre [lrange $pre 0 end-1] 
        if {[catch {
            set path [file join [file nativename "~$user"] [lindex $param 1]]
        }]} {
            return {0 0}
        }
        foreach pp $pre {
            set path [file join $path $pp]
        }
        return [list 1 1 $path]
    }  else  {
        return {0 0}
    }
}
# }}}
