ad_page_contract { 
    Check XQL files for internal consistency 
    and file manifest etc.

    Output streamed. 

    @author Jeff Davis (davis@xarg.net)
    @creation-date 2002-08-12
    @cvs-id $Id: xql.tcl,v 1.3 2002/10/14 11:10:22 davis Exp $
} { 
    {package:trim,optional {}}
}
    

ReturnHeaders

# couple of local helper procs 
proc ::xql_p {file} { 
    return [expr [string match {*.xql} $file] || [file isdirectory $file]]
}

proc bel {file} {
    return "<b>$file</b>(<a href=\"e?file=$file\">edit</a>)"
}

set startdir [acs_root_dir]/packages/$package

ns_write "Checks starting from $startdir<br /><br />"

ns_write "Checking that the XQL files are well formed...<br /><br />"
set checked 0

foreach file [ad_find_all_files -check_file_func ::xql_p $startdir] { 
    incr checked 
    
    # Check that the file parses as xml
    set fp [open $file "r"]
    set data [read $fp]
    close $fp

    # Fix up the queries (note that .xql files are not XML since 
    # we let people put < and & etc in the queries unquoted.

    set data [db_qd_internal_prepare_queryfile_content $data]

    if { [catch {set parse [xml_parse -persist $data]} errMsg] } { 
        ns_write "XML Parse Error: [bel $file]<br /><pre>[ad_quotehtml $errMsg]</pre><br />\n"
    } else { 
        # lets walk the nodes and check they are what we want to see.

        # We are done so just let it go man.
        xml_doc_free $parse
    }

    # Errors:
    #   .xql files without .tcl 
    #   dbname not blank or postgresql or oracle
    #   -oracle w/o generic or -postgresql
    #   -postgresql w/o generic or -oracle
    #  

    regexp {(.*)[.]xql$} $file match base
    set onexql($base) 1 
    
    if {![file exists ${base}.tcl] && ![file exists ${base}.vuh]} { 

        # the file did not exist so we must have a -db extension...
        regexp {(.*?)(-)?([A-Za-z_]*)[.]xql$} $file match base dummy db
        if { ![empty_string_p $db] 
             && ![string match $db oracle]
             && ![string match $db postgresql] } { 
            ns_write "bad db $db file [bel $file]<br />\n"
        } elseif { ![empty_string_p $db] 
                   && ![regexp $db $data] } { 
            ns_write "rdbms $db missing [bel $file]<br />\n"
            
        } elseif {[empty_string_p $db]
                  && [regexp {<rdbms>} $data] } { 
            ns_write "rdbms found in generic [bel $file]<br />\n"
        } 
          
        if {[string equal $db postgresql] || [empty_string_p $db]} { 
            if {[regexp {(nvl[ ]*\(|decode[ ]*\()} $data]} { 
                ns_write "postgres or generic with oracle code [bel $file]<br />\n"
            }
        } else { 
            if {[regexp {(now[ ]*\()} $data] || [empty_string_p $db]} { 
                ns_write "oracle or generic with oracle code [bel $file]<br />\n"
            }
        }
        
    }
    set allxql($base) 1
}

ns_write "<br />Checked $checked .xql files.<br /><br />"


ns_write "Checking file existence...<br /><br />"

foreach xql [array names allxql] { 
     
    # check there is a corresponding .tcl file
    if {![file exists ${xql}.tcl] 
        && ![file exists ${xql}.vuh]} {
        ns_write "missing .tcl or .vuh file for <b>${xql}</b><br />\n"
    } 

    # check that if there is a db specific version that the corresponding 
    # generic or other db file exists...
    if {[info exists onexql(${xql}-oracle)] 
          && !([info exists onexql(${xql}-postgresql)] 
               || [info exists onexql(${xql})]) } {
        ns_write "No postgresql or generic <b>$xql</b><br />\n"
    }
    if {[info exists onexql(${xql}-postgresql)] 
          && !([info exists onexql(${xql}-oracle)] 
               || [info exists onexql(${xql})]) } {
        ns_write "No oracle or generic <b>$xql</b><br />\n"
    }
    
}
ns_write "Done checking file existence.<br /><br />"


