package provide nstclhttpd::nsset 1.0
package require nstclhttpd::core

namespace eval ns {}
namespace eval ns::set {}

if {![info exists ns::set::keyId]} {
    set ns::set::keyId 0
}

proc ns::set::new {name ctxList} {
    variable keyId
    variable setval
    variable setname
    variable ctx
    variable cset

    set id [incr keyId]
    lappend ctxList set.$id
    set setval($id) [list]
    set setname($id) $name
    set cset($id) $ctxList

    foreach c $ctxList {
        lappend ctx($c) $id
    }
    return $id
}

proc ns::set::cleanup {c} {
    variable setval
    variable setname
    variable ctx
    variable cset

    foreach id $ctx($c) {
        set cs $cset($id)
        unset cset($id)
        unset setval($id)
        unset setname($id)
        foreach cname $cs {
            set idx [lsearch -exact $ctx($cname) $id]; set ctx($cname) [lreplace $ctx($cname) $idx $idx]
            if {[llength $ctx($cname)]==0} {
                unset ctx($cname)
            }
        }
    }
}

proc ns::set::getname {id} {
    variable setname
    return $setname($id)
}
proc ns::set::setname {id n} {
    variable setname
    set setname($id) $n
    return $n
}
proc ns::set::seteval {id var {cmd ""}} {
    variable setval
    upvar $var v
    set v $setval($id)
    if {[catch {uplevel 1 $cmd} rc]==1} {
        return -code error -errorinfo $::errorInfo -errorcode $::errorCode $rc
    }
    set setval($id) $v
    return $id
}

namespace eval ns::ns_set {}

ns::nscmd ::ns_set ns::ns_set::_ copy cput create delete delkey find free get icput idelkey ifind iget isnull iunique key merge move name new print put size split truncate unique update value 

ad_proc ns::ns_set::_copy {{-persist:boolean} setId } {
    if {$persist_p} {
        set cl [list global]
    }  else  {
        set cl [list $::ns::sock]
    }
    set cl [ns::set::new [ns::set::getname $setId] $cl]
    ns::set::seteval $setId v0 {}
    ns::set::seteval $cl v {set v $v0}
    return $cl
}
ad_proc ns::ns_set::_cput {setId key value} {
    set ok 1
    set rc -1
    ns::set::seteval $setId v {
        foreach {kn kv} $v {
            if {[string equal $kn $key]} {
                set ok 0
                break
            }
        }
        if {$ok} {
            lappend v $key $value
            set rc [expr {[llength $v]/2-1}]
        }
    }
    return $rc
}
ad_proc ns::ns_set::_create {{-persist:boolean} {name ""}} {
    if {$persist_p} {
        set cl [list global]
    }  else  {
        set cl [list $::ns::sock]
    }
    return [ns::set::new $name $cl]
}
ad_proc ns::ns_set::_delete {setId fieldNumber} {
    ns::set::seteval $setId v {
        set v [lreplace $v [expr {$fieldNumber*2}] [expr {$fieldNumber*2+1}]]
    }
}
ad_proc ns::ns_set::_delkey {setId key} {
    ns::set::seteval $setId v {
        set idx 0
        foreach {kn kv} $v {
            if {[string equal $kn $key]} {
                set v [lreplace $v $idx [expr {$idx+1}]]
                break
            }
            incr idx 2
        }
    }
    return ""
}
ad_proc ns::ns_set::_find {setId key} {
    set rc -1
    set idx 0
    ns::set::seteval $setId v {
        foreach {kn kv} $v {
            if {[string equal $kn $key]} {
                set rc $idx
                break
            }
            incr idx
        }
    }
    return $rc
}
ad_proc ns::ns_set::_free {setId} {
    ns::set::cleanup set.$setId
}
ad_proc ns::ns_set::_get {setId key} {
    set rc ""
    ns::set::seteval $setId v {
        foreach {kn kv} $v {
            if {[string equal $kn $key]} {
                set rc $kv
                break
            }
        }
    }
    return $rc
}
ad_proc ns::ns_set::_icput {setId key value} {
    set ok 1
    set rc -1
    ns::set::seteval $setId v {
        foreach {kn kv} $v {
            if {[string equal $kn $key]} {
                set ok 0
                break
            }
        }
        if {$ok} {
            lappend v $key $value
            set rc [expr {[llength $v]/2-1}]
        }
    }
    return $rc
}
ad_proc ns::ns_set::_idelkey {setId key} {
    ns::set::seteval $setId v {
        set idx 0
        foreach {kn kv} $v {
            if {[string equal -nocase $kn $key]} {
                set v [lreplace $v $idx [expr {$idx+1}]]
                break
            }
            incr idx 2
        }
    }
    return ""
}
ad_proc ns::ns_set::_ifind {setId key} {
    set rc -1
    set idx 0
    ns::set::seteval $setId v {
        foreach {kn kv} $v {
            if {[string equal -nocase $kn $key]} {
                set rc $idx
                break
            }
            incr idx
        }
    }
    return $rc
}
ad_proc ns::ns_set::_iget {setId key} {
    set rc ""
    ns::set::seteval $setId v {
        foreach {kn kv} $v {
            if {[string equal -nocase $kn $key]} {
                set rc $kv
                break
            }
        }
    }
    return $rc
}
ad_proc ns::ns_set::_isnull {setId fieldNumber} {
    return 0
}
ad_proc ns::ns_set::_iunique {setId key} {
    set rc 0
    ns::set::seteval $setId v {
        foreach {kn kv} $v {
            if {[string equal -nocase $kn $key]} {
                incr rc
            }
        }
    }
    return [expr {$rc>1?0:1}]
}
ad_proc ns::ns_set::_key {setId fieldNumber} {
    ns::set::seteval $setId v {
        set rc [lindex $v [expr {$fieldNumber*2+0}]]
    }
    return $rc
}
ad_proc ns::ns_set::_merge {high low} {
    error "Not implemented"
}
ad_proc ns::ns_set::_move {to from} {
    error "Not implemented"
}
ad_proc ns::ns_set::_name {setId} {
    return [ns::set::getname $setId]
}
ad_proc ns::ns_set::_new {{-persist:boolean} {name ""}} {
    if {$persist_p} {
        set cl [list global]
    }  else  {
        set cl [list $::ns::sock]
    }
    return [ns::set::new $name $cl]
}
ad_proc ns::ns_set::_print {setId} {
    set max 0
    ns::set::seteval $setId v {}
    foreach {kn kv} $v {
        set sl [string length $kn]
        if {$sl>$max} {set max $sl}
    }
    set rc "Set \"[ns::set::getname $setId]\":\n"
    foreach {kn kv} $v {
        append rc [format "%-${max}s = \"%s\"\n" $kn $kv]
    }
    return $rc
}
ad_proc ns::ns_set::_put {setId key value} {
    ns::set::seteval $setId v {
        lappend v $key $value
        set rc [expr {[llength $v]/2-1}]
    }
    return $rc
}
ad_proc ns::ns_set::_size {setId} {
    ns::set::seteval $setId v {
        set rc [expr {[llength $v]/2}]
    }
    return $rc
}
ad_proc ns::ns_set::_split {{-persist:boolean} setId {splitChar .}} {
    error "Not implemented"
}
ad_proc ns::ns_set::_truncate {setId size} {
    ns::set::seteval $setId v {
        set v [lrange $v 0 [expr {$size*2-1}]]
    }
    return ""
}
ad_proc ns::ns_set::_unique {setId key} {
    set rc 0
    ns::set::seteval $setId v {
        foreach {kn kv} $v {
            if {[string equal $kn $key]} {
                incr rc
            }
        }
    }
    return [expr {$rc>1?0:1}]
}
ad_proc ns::ns_set::_update {setId key value} {
    ns::set::seteval $setId v {
        set idx 1
        foreach {kn kv} $v {
            if {[string equal $kn $key]} {
                set v [lreplace $v $idx $idx $value]
                set idx -1
                break
            }
            incr idx 2
        }
        if {$idx>0} {lappend v $key $value}
    }
}
ad_proc ns::ns_set::_value {setId fieldNumber} {
    ns::set::seteval $setId v {
        set rc [lindex $v [expr {$fieldNumber*2+1}]]
    }
    return $rc
}

