package provide nstclhttpd::url 1.0
package require nstclhttpd::core

package require ncgi

namespace eval ns {}
namespace eval ns::url {}

if {![info exists ns::url::registered_proc]} {
    set ns::url::registered_proc [list]
}
if {![info exists ns::url::registered_filter.preauth]} {
    set ns::url::registered_filter.preauth [list]
}
if {![info exists ns::url::registered_filter.postauth]} {
    set ns::url::registered_filter.postauth [list]
}
if {![info exists ns::url::registered_filter.trace]} {
    set ns::url::registered_filter.trace [list]
}

proc ns::url::urlsplit {url} {
    set rc [list]
    foreach e [split $url /] {
        set e [ns_urldecode $e]
        if {$e==".."} {set rc [lrange $rc 0 end-1]}
        if {($e==".")||($e=="..")||([string index $e 0]=="~")} {continue}
        if {[string length $e]>0} {
            lappend rc $e
        }
    }
    return $rc
}

proc ns::url::parts {url} {
    return [urlparts [urlsplit $url]]
}
proc ns::url::urlparts {surl} {
    set idx 0
    foreach e $surl {
        if {([string first * $e]>=0)||([string first ? $e]>=0)} {
            break
        }
        incr idx
    }
    set rcp [lrange $surl 0 [expr {$idx-1}]]
    set rcs [lrange $surl [expr {$idx-0}] end]
    return [list $rcp $rcs]
}

# match order

proc ns::url::compareweight {d0 d1} {
    set in0 [lindex $d0 1]
    set in1 [lindex $d1 1]
    set d0 [lindex $d0 2]
    set d1 [lindex $d1 2]

    set pre0 [lindex $d0 0]
    set suf0 [lindex $d0 1]
    set pre1 [lindex $d1 0]
    set suf1 [lindex $d1 1]

    set rc [expr {[llength $pre0]-[llength $pre1]}]
    if {$rc==0} {
        foreach s0 $pre0 s1 $pre1 {
            set rc [expr {[string length $s0]-[string length $s1]}]
            if {$rc!=0} {break}
            set rc [string compare $s0 $s1]
            if {$rc!=0} {break}
        }
    }
    if {$rc==0} {
        set rc [expr {[llength $suf0]-[llength $suf1]}]
    }
    if {$rc==0} {
        foreach s0 $suf0 s1 $suf1 {
            set rc [expr {[string length $s0]-[string length $s1]}]
            if {$rc!=0} {break}
            set rc [string compare $s0 $s1]
            if {$rc!=0} {break}
        }
    }
    if {$rc==0} {
        return [expr {$in1-$in0}]
    }
    return $rc
}

# prefix-suffix based matching

proc ns::url::pmatch {ppat purl inherit} {
    set pp [lindex $ppat 0]
    set ps [lindex $ppat 1]
    if {[llength $purl]<[llength $pp]} {return 0}
    if {(!$inherit)&&([llength $purl]!=([llength $ps]+[llength $pp]))} {return 0}
    for {set i 0} {$i<[llength $pp]} {incr i} {
        if {![string equal [lindex $pp $i] [lindex $purl $i]]} {
            return 0
        }
    }
    set idx [llength $pp]
    set len [llength $purl]
    if {($idx+[llength $ps])>$len} {
        return 0
    }
    foreach e $ps {
        while {($idx<$len)&&(![string match $e [lindex $purl $idx]])} {
            incr idx
        }
    }
    if {($idx>=$len)&&($len>[llength $pp])} {
        return 0
    }
    return 1
}

# proc/filter/trace registering

proc ns::url::register {type inherit url procname ar} {
    variable registered_$type; upvar 0 registered_$type re
    unregister $type $inherit $url

    lappend re [list $url $inherit [parts $url] $procname $ar]

    set re [lsort -decreasing -command ns::url::compareweight $re]
}
proc ns::url::unregister {type inherit url} {
    variable registered_$type; upvar 0 registered_$type re
    set idx 0; foreach e $re {
        if {([lindex $e 1]==$inherit)&&([lindex $e 0]==$url)} {
            set re [lreplace $re $idx $idx]
            return
        }
        incr idx
    }
}

proc ns::url::matchurl {type url} {
    set surl [urlsplit $url]
    variable registered_$type; upvar 0 registered_$type re
    set rc [list]
    set idx 0; foreach e $re {
        if {[pmatch [lindex $e 2] $surl [lindex $e 1]]} {
            lappend rc $e
        }
    }
    return $rc
}

# AOLserver emulation

ad_proc ns_register_proc {{-noinherit:boolean} method url procname args} {
    ns::url::register proc [expr {$noinherit_p?0:1}] $url $procname $args
}
ad_proc ns_unregister_proc {{-noinherit:boolean} method url} {
    ns::url::unregister proc [expr {$noinherit_p?0:1}] $url
}

ad_proc ns_register_filter {{-noinherit:boolean} when method url procname args} {
    ns::url::register filter.$when [expr {$noinherit_p?0:1}] $url $procname $args
}
ad_proc ns_unregister_filter {{-noinherit:boolean} when method url} {
    ns::url::unregister filter.$when [expr {$noinherit_p?0:1}] $url
}

ad_proc ns_url2file {url} {
    global Config
    return [eval [concat file join [list $Config(docRoot)] [ns::url::urlsplit [string map [list \\ / : /] $url]]]]
}

ad_proc ns_guesstype {path} {
    return [Mtype $path]
}

proc ns_urlencode {str} {
    return [ncgi::encode [encoding convertto $::ns::encoding $str]]
}
proc ns_urldecode {str} {
    return [encoding convertfrom $::ns::encoding [ncgi::decode $str]]
}

