package provide tclwebtest 0.1

namespace eval tclwebtest {

# source the entities lib file
source "[file dirname [info script]]/entities.tcl"

# do we print debugging msgs that are in this file?
variable DEBUG_LIB_P 0

set VERSION "0.1d"

# shell the html checker tidy be invoked on each result page?
variable TIDY 0


# follow dirty redirects by default?
variable FOLLOWEQUIV 0

namespace export do_request reset_session debug response cookies assert assertion_failed link form field translate_entities known_bug



proc http_namespace { } {
    if { [llength [namespace children ::tclwebtest ::tclwebtest::http]] > 0 } {
        return "::tclwebtest::http"
    } else {
        return "::http"
    }
}


proc initialize_variables {} {
    # Important: every variable command here must also set a value,
    # otherwise reset won't work as expected and cause ugly and subtle
    # failures.

    variable url ""
    variable http_status ""
    variable body ""
    variable body_without_comments ""

    # All visible text of this page
    variable text ""

    
    # A list that contain alternating cookie name and value. Value is
    # a list by itself, containing array key and value pairs. This is
    # different from the format of the other lists, e.g. links
    variable cookies [list]


    # not implemented yet
    variable framed_p ""
    variable frames_name ""
    variable frames_body ""

    # has the corresponding proc already been called after the current
    # request?
    variable links_extracted_p 0

    # A list of lists. The inner list contains a key/value list
    # prepared for array set; with those keys: full, url, content
    variable links [list]


    # has the corresponding proc already been called after the current
    # request?
    variable forms_extracted_p 0


    # Again a list of lists, where the inner list is an array
    # list. Keys: full, action, method, content, fields. Fields is a
    # list of array lists, that contains those keys: full, name, type,
    # value, caption, choices

    # caption is what appears before the field, e.g. "First name:"

    # choices is a list of possible values when field is of type
    # select, radio or checkbox. For checkbox it will always contain
    # an empty string and the value of the "value" attribute

    variable forms [list]

    # Indices of the current active form and field.
    variable current_form ""
    variable current_field ""
    variable field_modified_p 0
    variable current_link ""

}

initialize_variables


# set variables to reasonable values before the first request
set user_agent "tclwebtest/$VERSION"
set url ""
 
set cookies [list]

# Links all variables defined in the named block into the calling
# context. Meant to be used at the beginning of procs. Dangerous,
# because suddenly a lot of variables exist in the procs context. Used
# to do more, now it's just for the regex variables. (sort of useless)
proc link_variables { block_name } {

    set to_eval "uplevel \{
                   variable $block_name
                   eval \$$block_name 
                 \}"

    eval $to_eval
}



proc reset_session { } {
    namespace eval ::tclwebtest:: initialize_variables
}


# Emit the message to stdout, even if logging output is redirected to
# a file. Intended to be used when writing tests. Its only advantage
# over using puts is that it does not have to be deleted when writing
# the test is finished.
proc debug args {
    parse_args $args { -lib } { msg }

    if { $lib } {
        # debugging message from the tclwebtest code itself
        if { [info exists ::tclwebtest::DEBUG_LIB_P] && $::tclwebtest::DEBUG_LIB_P } {
            puts "DEBUG LIB: $msg"
        }
    } else {
        # normal debugging message. TODO implement a -debug switch,
        # currently it's always on
        puts "DEBUG: $msg"
    }
}


proc log msg {
    variable log_channel

    if { ![info exists log_channel] || $log_channel == "" } {
        set log_channel stdout
    }

    puts $log_channel $msg

}

# Search for the first link in the extracted links that matches the
# given criteria and return its array list. If there is no link that
# matches throws an assertion_failed.

# TODO implement current_link similar to work like current_field
# (esp. when no further arguments are given, e.g. like this "link
# follow")

proc link args {
       
    set command [lindex $args 0]
    set args [lrange $args 1 end]
    
    extract_links_if_necessary

    switch -glob $command {
        find {
            # find a link with the given attributes or the first one
            # and return the full list or the index. TODO maybe should
            # deal with the -fail switch too.

            parse_args $args { -index -next -fail } args

            
            # We need a way to loop through all links.This assumes
            # that somehow current_link will be set to -1 for a full
            # search through all links and is somewhat different to
            # the field behaviour
            if { $next } {
                # find the next link that matches after the
                # current_link
                
                # fail if current_link points to the last link already
                if { [expr $::tclwebtest::current_link + 1] >= [llength $::tclwebtest::links] } {
                    if { $fail } {
                        return
                    } else {
                        assertion_failed "No more links, thus \"link find -next\" failed. "
                    }
                }
                
                incr ::tclwebtest::current_link
                set offset $::tclwebtest::current_link
                debug -lib "next is true, offset is $offset"
            } else {
                # find the first that matches
                set offset 0
            }

            # do something if called without args
            if { [llength $args] == 0 } {
                if { [llength $::tclwebtest::links] == 0 } {
                    if { $fail } {
                        return
                    } else {
                        assertion_failed "There are no links"
                    }
                } else {
                    if { !$next } {
                        # "link find" called without -next and without
                        # arguments
                        if { $::tclwebtest::current_link == -1 } {
                            # set to first link
                            set ::tclwebtest::current_link 0
                        }
                    }
                    if $index {
                        return $::tclwebtest::current_link
                    } else {
                        return [lindex $::tclwebtest::links $::tclwebtest::current_link]
                    }
                }
            }

            set list_to_search [lrange $::tclwebtest::links $offset end]

            set found_idx [find_in_array_list -index $list_to_search [list ~c content ~u url ~f full] $args]

            if { $found_idx == "" } {
                if { $fail } {
                    return
                } else {
                    assertion_failed "No link found that matches '$args'"
                }
            } else {
                set found_idx [expr $found_idx + $offset]
                set ::tclwebtest::current_link $found_idx

                if { $fail } {
                    # found something while -fail was set
                    assertion_failed "'link find -fail $args' did not expect to find a link, yet it found this one: [lindex $::tclwebtest::links $found_idx]"
                } else {
                    if $index { return $found_idx } else {
                        return [lindex $::tclwebtest::links $found_idx] 
                    }
                }
            }
        }
        
        reset_current {
            # see "field find -next" above
            set ::tclwebtest::current_link -1
        }

        follow {
            # just a shortcut for "do_request [link find args]"
            set evalstr "do_request \[link get_url $args\]"
            debug -lib "following a link by: $evalstr"
            eval $evalstr
        }

        current {
            if { $::tclwebtest::current_link == -1 } {
                # set to the first link. will scream if there are no
                # links
                link find
                debug -lib "resetting current_link"
            }

            debug -lib "current_link is $::tclwebtest::current_link"

            return [lindex $::tclwebtest::links $::tclwebtest::current_link]
        }
        
        all {
            # return all links. TODO maybe should allow for
            # expressions to limit the returned list
            return $::tclwebtest::links
        }

        get_* {
            # return the specified attribute of a link. TODO write a
            # selftest for this
            parse_args $args { } { args }
            
            if { ![regexp {get_(.+)} $command match attribute_name] } {
                error "In field get_*: command $command is not valid"
            }
            
            # it's called content but some might want to call it text,
            # which sounds better
            if { $attribute_name == "text" } { set attribute_name "content" }

            if { [llength $args] > 0 } {
                eval "link find $args"
            }
            
            array set a_link [link current]

            if { [lsearch [array names a_link] $attribute_name] == -1 } {
                error "In link $command: $attribute_name not found in array, we only have: [array names a_link]"
            }

            return $a_link($attribute_name)
        }

        default {
            error "The command \"link $command\" does not exist"
        }
    }
}


# Only one arg: arg is interpreted as condition that should return
# true. Typically condition is a comparison, like assert { $val ==
# "foo" }.

# several convenience commands like text, full, status etc. are
# provided. E.g. "assert text bla" tests for the occurence of "bla" in
# the http response text.

# With -fail assert expects the condition to be false.

# Although it seems quite useful the user_explanation idea of the acs
# testharness was dropped again, simply because it just does not fit
# in the complicated api here /read: laziness. It is assumed that
# programmers don't like to write textual explanations of asserts
# anyway. Blabla. TODO: for "assert text" it seems useful to
# reintroduce it ...
proc assert args {
    
    parse_args $args { -fail } args

    if { [llength $args] == 0 } {
        error "assert must be called with at least one argument" 
    }

    if { [llength $args] == 1 } {
        set command "condition"
    } else {
        set command [lindex $args 0]
        set args [lrange $args 1 end]
    }
    
    if { $command == "body" } { set command "full" }

    switch $command {

        condition {
            # Sets the variable _assert_p in the calling
            # context. Don't know how else i could evaluate a
            # condition in the same way that if {...} does

            if $fail {
                # assert has been called with "assert", not "assert
                # condition", so -fail might have been parsed already.
                parse_args $args { } condition
            } else {
                parse_args $args { -fail } condition
            }
            set to_eval "if \{ $condition \} \{ set _assert_p 1 \} else \{ set _assert_p 0 \}"
            uplevel $to_eval
            upvar _assert_p _assert_p
            if { $_assert_p == $fail } {
                set condition [string trim $condition]
                set extra [list]

                # Heuristically parse the condition for all variables
                # ($a, $b ...), upvar them and append their values to
                # the message. Cannot do execution of [...] blocks
                # (would propably have unforeseeable side effects)
                set condition_to_search $condition
                regsub -all {\[.*?\]} $condition {} condition_to_search
                foreach word [split $condition_to_search {$}] {
                    if { $word == "" || [string first $word $condition_to_search] == 0 } {
                        # the first result from the split is the text
                        # from the beginning to the first $
                        continue
                    }
                    if [regexp {(.*?)(\s|$)} $word match varname] {
                        debug -lib "displaying the value of '$varname'"
                        upvar $varname value
                        if { [string length $value] > 30 } {
                            set value "[string range $value 0 26]..."
                        }
                        lappend extra "\$$varname: $value"
                    }
                }

                if $fail {
                    assertion_failed "Assertion \"$condition\" did not fail, but -fail was given"
                } else {
                    assertion_failed "Assertion \"$condition\" failed. [join $extra "; "]"
                }
            }
        }

        text {
            # Test an expression against the visible text of the
            # result page. TODO make a more meaningful message
            parse_args $args { -fail } search_expr
            if $fail {
                assert -fail { [regexp -nocase $search_expr $::tclwebtest::text] }
            } else { 
                assert { [regexp -nocase $search_expr $::tclwebtest::text] }
            }
        }

        full {
            # Test an expression against the full html source of the
            # result page.
            parse_args $args { -fail } search_expr
            if $fail {
                assert -fail { [regexp -nocase $search_expr $::tclwebtest::body] }
            } else {
                assert { [regexp -nocase $search_expr $::tclwebtest::body] }
            }
        }

        default {
            error "The command \"assert $command\" does not exist."
        }
    }
} ;# end of assert


# from /acs-test-harness/tcl/test-procs.tcl
proc assertion_failed { assertionMsg } {
    error "$assertionMsg" "$assertionMsg\n--- end of assertionMsg ---\n" { NONE }
}


# current - get or set the index of the current form. TODO complain
# when called with non-existing command
proc form args {
    extract_forms_if_necessary

    set command [lindex $args 0]
    set remaining_args [lrange $args 1 end]

    if { $command == "find" } {
        # find - set currentform to the first form or to a form on the page
        # with specified criteria

        # TODO -next

        parse_args $remaining_args { } args

        if { [llength $::tclwebtest::forms] == 0 } {
            # no forms present
            assertion_failed "No form present"
        } elseif { [llength $args] == 0 } {
            # set to first form on this page
            set ::tclwebtest::current_form 0
            return
        }

        set found [find_in_array_list -index $::tclwebtest::forms [list ~c content ~a action ~m method ~n name ~f full] $args]

        if { $found == "" } {
            assertion_failed "No form with found with search_string $args"
        }
        set ::tclwebtest::current_form $found
        set ::tclwebtest::current_field -1 ;# TODO really?
        return
    }

    if { ![info exists ::tclwebtest::current_form] || $::tclwebtest::current_form == -1 } {
        # setting to the first form
        #puts "form calls itself"
        form find
    }

    if { [set current_form $::tclwebtest::current_form] == -1 } {
        error "No form found at all"
    }
    
    switch -glob $command {

        submit {
            array set a_form [form current]
            set query_key_values [list]

            foreach field $a_form(fields) {
                # add all form fields to the POST query except the
                # submit button

                catch { unset a_field }
                array set a_field $field

                if { [string match $a_field(type) "select"] } {
                    # a select field - special format of value (it's a
                    # list)
                    foreach select_value $a_field(value) {
                        lappend query_key_values $a_field(name) $select_value
                    }
                } elseif { ![string match $a_field(type) "submit"] } {
                    lappend query_key_values $a_field(name) $a_field(value)
                }
            }

            debug -lib "POSTING: $query_key_values"
            do_request $a_form(action) $query_key_values
        }
        
        current {
            # just return the current form
            parse_args $remaining_args { -index } { }
            if $index {
                return $::tclwebtest::current_form
            } else {
                return [lindex $::tclwebtest::forms $::tclwebtest::current_form]
            }
        }

        get_* {
            # return the specified attribute of a form. TODO write a
            # selftest for this
            parse_args $args { } { args }
            
            if { ![regexp {get_(.+)} $command match attribute_name] } {
                error "In form get_*: command $command is not valid"
            }

            array set a_form [form current]

            if { [lsearch [array names a_form] $attribute_name] == -1 } {
                error "In form $command: $attribute_name not found in array, we only have: [array names a_form]"
            }

            return $a_form($attribute_name)
        }

        all {
            # return a list of all forms
            return $::tclwebtest::forms
        }

        default {
            error "the command \"form $command\" does not exist"
        }
    }
}

# We keep a pointer to the current field. It will initially (after
# first access to any form or field) be set to index 0. The command
# "field find" and all commands that call it will set that pointer to
# the index of the found field. Additionally we keep a flag
# field_modified_p, that will be set by all commands that modify the
# field value, and unset by a find operation.

# TODO add a ~i attribute for the index of the field.
proc field args {
    set command [lindex $args 0]
    set args [lrange $args 1 end]

    extract_forms_if_necessary

    if { $::tclwebtest::current_form == -1 } {
        form find
    }

    set form [lindex $::tclwebtest::forms $::tclwebtest::current_form]
    array set a_form $form
    
    switch -glob $command {
        fill {
            # Fill the specified text in the first fillable field
            # (text, textarea, password).
            
            # This just updates the field(value), not the actual html
            # code in field(full)
            
            parse_args $args { } { value args }
            find_field_of_type { text textarea password } $args
                        
            set cf $::tclwebtest::current_field   ;# just a shortcut
            
            set field [lindex $a_form(fields) $cf]
            array set a_field $field
            set a_field(value) $value
            set field [array get a_field]
            set a_form(fields) [lreplace $a_form(fields) $cf $cf $field]
            set form [array get a_form]
            set ::tclwebtest::forms [lreplace $::tclwebtest::forms $::tclwebtest::current_form $::tclwebtest::current_form $form]
            
            set ::tclwebtest::field_modified_p 1
        } ;# end of fill
        
        fill_hidden {
            # Fill the specified text in the first hidden field. 
            
            # This just updates the field(value), not the actual html
            # code in field(full) and is a copy of fill above
            
            parse_args $args { } { value args }
            find_field_of_type { hidden } $args
                        
            set cf $::tclwebtest::current_field   ;# just a shortcut
            
            set field [lindex $a_form(fields) $cf]
            array set a_field $field
            set a_field(value) $value
            set field [array get a_field]
            set a_form(fields) [lreplace $a_form(fields) $cf $cf $field]
            set form [array get a_form]
            set ::tclwebtest::forms [lreplace $::tclwebtest::forms $::tclwebtest::current_form $::tclwebtest::current_form $form]
            
            set ::tclwebtest::field_modified_p 1
        } ;# end of fill_hidden

        select {
            # field select ?-index? value ?search_args ...?
            
            # To be used with radio buttons or select fields. Select
            # the entry that contains the specified text in its
            # caption or the nth entry by specifying -index and a
            # number as value (starting from 0). If the field is a
            # select with multiple selections, then the new value will
            # replace previously selected ones.
            
            parse_args $args { -index } { search_arg args }
            
            find_field_of_type { radio select } $args
            
            array set a_field [lindex $a_form(fields) $::tclwebtest::current_field]
            if { $index } {
                # was called with -index
                if { $search_arg > [expr [llength $a_field(choices)] - 1] } {
                    assertion_failed "This field (name: $a_field(name)) cannot be set to index $value. Its choices are: $a_field(choices)"
                }
                set a_field(value) [lindex $a_field(choices) $search_arg]
            } else {
                # not called with -index, e.g. an expression is given
                # for the desired caption to set
                
                # TODO we might allow for search args such as { ~v
                # some-explicit-value } instead of just searching in
                # caption
                
                debug -lib "SEARCHING FOR: $search_arg"
                set found_p 0
                foreach choice $a_field(choices) {
                    set looped_value [lindex $choice 0]
                    set looped_caption [lindex $choice 1]
                        
                    if { [regexp -nocase $search_arg $looped_caption] } {
                        set found_p 1
                        debug -lib "FOUND: $choice"
                        set a_field(value) [list $looped_value]
                        break
                    }
                }
                    
                if !$found_p {
                    assertion_failed "This field has no choice $search_arg. It's only offerings are: $a_field(choices)"
                }
            }
            replace_current_field [array get a_field]

            set ::tclwebtest::field_modified_p 1
        } ;# end of select

        multiselect {
            parse_args $args { } { value_list args }
            find_field_of_type select [concat ~m 1 $args]
            array set a_field [lindex $a_form(fields) $::tclwebtest::current_field]
            
            foreach subex $value_list {
                # loop through all search args (when this is not a
                # multiple select then there will only be one)
                debug -lib "SEARCHING FOR SUBEX: $subex"
                set found_p 0
                
                foreach choice $a_field(choices) {
                    set looped_value [lindex $choice 0]
                    set looped_caption [lindex $choice 1]
                    
                    if { [regexp -nocase $subex $looped_caption] } {
                        set found_p 1
                        if { $a_field(type) == "select" && $a_field(multiple_p) } {
                            # a multiple selection - add the found
                            # value to the existing value list
                            lappend a_field(value) $looped_value
                        } else {
                            # no multiple selection allowed - just set the value
                            set a_field(value) [list $looped_value]
                        }
                        break
                    }
                    
                } ;# next choice
                
                if !$found_p {
                    assertion_failed "This field has no choice $subex. It's only offerings are: $a_field(choices)"
                }
            } ;# next subex
            
            replace_current_field [array get a_field]
            set ::tclwebtest::field_modified_p 1
        } ;# end of multiselect
        
        deselect {
            # clear a multiple select field. TODO 
            find_field_of_type select [concat ~m 1 $args]
            
            array set a_field [field current]
            
            if { ! ($a_field(type) == "select" && $a_field(multiple_p)) } {
                assertion_failed "You cannot deselect the field $a_field(name) because it is not a multiple select field"
            }
            set a_field(value) [list]
            replace_current_field [array get a_field]
            set ::tclwebtest::field_modified_p 1
        } ;# end of deselect
        
        check {
            # field check ?search_args ...?
            
            # Check a checkbox
            
            find_field_of_type checkbox $args
            
            array set a_field [field current]
            
            if { $a_field(type) != "checkbox" } {
                assertion_failed "This field is not a checkbox"
            }
            
            set a_field(value) [lindex $a_field(choices) 1]
            replace_current_field [array get a_field]
            set ::tclwebtest::field_modified_p 1
        } ;# end of select
        
        uncheck {
            # opposite of field check
            find_field_of_type checkbox $args
            
            array set a_field [field current]
            
            if { $a_field(type) != "checkbox" } {
                assertion_failed "This field is not a checkbox"
            }
            
            set a_field(value) [lindex $a_field(choices) 0]
            replace_current_field [array get a_field]
            set ::tclwebtest::field_modified_p 1
        } ;# end of uncheck
        
        get_* {
            # return the specified attribute of a field. TODO write a
            # selftest for this
            parse_args $args { } { args }
            
            if { ![regexp {get_(.+)} $command match attribute_name] } {
                error "In field get_*: command $command is not valid"
            }

            if { [llength $args] > 0 } {
                eval "field find $args"
            }
            
            array set a_field [field current]

            if { [lsearch [array names a_field] $attribute_name] == -1 } {
                error "In field $command: $attribute_name not found in array, we only have: [array names a_field]"
            }

            return $a_field($attribute_name)
        }

        current {
            # return or set the current field
            parse_args $args { -index } { { new_index "" } }
            
            if { $::tclwebtest::current_field == -1 } {
                field find
            }

            if { $new_index != "" } {
                # set the current field
                if { [expr [llength $a_form(fields)] - 1] < $new_index } {
                    assertion_failed "field current: Cannot set current field to index $new_index, field count is: [llength $a_form(fields)]"
                }

                set ::tclwebtest::current_field $new_index
                set ::tclwebtest::field_modified_p 0

            } else {
                # return the current field
                if $index {
                    return $::tclwebtest::current_field
                } else {
                    return [lindex $a_form(fields) $::tclwebtest::current_field]
                }
            }
        }

        all {
            # return all fields
            return $a_form(fields)
        }

        find {
            # Set the current_field to the first field that matches or
            # to the first field of the form if no criteria are given
            # TODO implement -fail
            parse_args $args { -next } { args }
            
            if { [llength $a_form(fields)] == 0 } {
                assertion_failed "there are no fields in the current form"
            }
            
            if { [llength $args] == 0 } {
                # no search criteria - set args to search for any
                # fillable field. Hidden fields cannot be manipulated
                # and thus cannot be found either. They are just
                # there.
                set args [list ~t (text|textarea|password|checkbox|radio|select)]
            }
            
            if { $next } {
                # search starts from the current field or from the one
                # after if the current has been modified already
                
                if { $::tclwebtest::field_modified_p } {
                    set offset [expr $::tclwebtest::current_field + 1]
                } else {
                    set offset $::tclwebtest::current_field
                }

                set list_to_search [lrange $a_form(fields) $offset end]
            } else {
                # search starts from the beginning
                set list_to_search $a_form(fields) 
                set offset 0
            }
            

            set found [find_in_array_list -index $list_to_search { ~c caption ~f full ~t type ~v value ~n name ~m multiple_p ~i index ~C choices } $args]
            if { $found == "" } {
                assertion_failed "Field not found. Args were: $args"
            } else {
                set ::tclwebtest::current_field [expr $offset + $found]
                set ::tclwebtest::field_modified_p 0
                return [lindex $a_form(fields) $::tclwebtest::current_field]
            }
        }
    }
} ;# end of field

# replace the field that current_field points to with the submitted
# field (which is supposed to be a list)
proc replace_current_field { new_field } {
    set cfo $::tclwebtest::current_form
    set cfi $::tclwebtest::current_field
    
    array set a_form [lindex $::tclwebtest::forms $cfo]
    set a_form(fields) [lreplace $a_form(fields) $cfi $cfi $new_field]
    
    set ::tclwebtest::forms [lreplace $::tclwebtest::forms $cfo $cfo [array get a_form]]

}

# Set the current field accordingly (used by the "field xxx" procs)
proc find_field_of_type { types search_args } {
    
    if { [string trim [join $search_args]] == "" } {
        set to_eval "field find -next ~t ([join $types "|"])"
    } else {
        set to_eval "field find ~t ([join $types "|"]) $search_args"
    }
    
    eval $to_eval
}


proc number_p { str } {
    return [regexp {^[-+]?[0-9]*(.[0-9]+)?$} $str]
}

proc wt_proc_valid_switch_p {str} {
    return [expr [string equal "-" [string index $str 0]] && ![number_p $str]]
}


proc strip_hyphen {str} {
    return [string range $str 1 end]
}

# unfortunately we do not want to use this:
# http://www.chevreux.org/projects_tcl.html because it does not seem
# to allow something like "command -someoption 5 somearg" but rather
# needs "command somearg -someoption 5", which is not what we want,
# especially for the commandline. So - rolling our own.

# allowed_switches is { ?allowed_switch? ... } and allowed_switch is {
# -option_name ?default? }. If only -option_name is specified then it
# will be interpreted as simple switch: 0 by default and 1 if given,
# and it is not possible to explicitely give it a value. When default
# is given then it is treated as option, and it is necessary to be
# followed by a value.

proc parse_args { args_list allowed_switches allowed_args } {
    set switch_list [list]

    # set all switches to 0 first
    foreach element $allowed_switches {
        
        # Get default value if present
        if { [llength $element] > 1 } {
            set allowed_switch [strip_hyphen [lindex $element 0]]
            set simple_switch_p($allowed_switch) 0
            set default($allowed_switch) [lindex $element 1]
        } else {
            set allowed_switch [strip_hyphen $element]
            set simple_switch_p($allowed_switch) 1
            set default($allowed_switch) 0
        }
        lappend switch_list $allowed_switch

        upvar $allowed_switch $allowed_switch
        set $allowed_switch $default($allowed_switch)
    }
    
    set switch_count 0

    for { set i 0 } { $i < [llength $args_list] } { incr i } {
        set arg [lindex $args_list $i]
        
        # If the argument doesn't begin with a hyphen, break.
        if { ![wt_proc_valid_switch_p $arg] } {
            break
        }
        
        # If the argument is "--", stop parsing for switches (but
        # bump up $i to the next argument, which is the first
        # argument which is not a switch).
        if { [string equal $arg "--"] } {
            incr i
            break
        }
        
        set allowed_switch [strip_hyphen $arg]
        
        if { [lsearch $switch_list $allowed_switch] == -1 } {
            error "$arg is not allowed. List of allowed switches: $switch_list"
        }
        
        incr switch_count

        upvar $allowed_switch $allowed_switch

        if { $simple_switch_p($allowed_switch) } {
            set $allowed_switch 1
        } else {
            # get the next arg as value for this option
            incr i
            set arg [lindex $args_list $i]
            set $allowed_switch $arg

            # this is badly worded ...
            incr switch_count
        }
    }
    
    
    # set the remaining args
    for { set i 0 } { $i < [llength $allowed_args] } { incr i } {
        set arg_definition [lindex $allowed_args $i]
        
        
        if { $arg_definition == "args" } {
            # the special args case
            # TODO complain if there are more allowed_args behind 
            # this definition
            upvar args args
            set args [lrange $args_list [expr $i + $switch_count] end]
            return
        }
        
        set arg [lindex $args_list [expr $i + $switch_count]]
        
        if { $i < [expr [llength $args_list] - $switch_count] } {
            # this value is specified
            
            if { [llength $arg_definition] > 1 } {
                set arg_name [lindex $arg_definition 0]
            } else {
                set arg_name $arg_definition
            }
            
            set _value [lindex $args_list [expr $i + $switch_count]]
        } else {
            # not specified - try to get the default value
            
            if { [llength $arg_definition] < 2 } { 
                error "$arg_definition must be specified"
            }
            
            set arg_name [lindex $arg_definition 0]
            set _value [lindex $arg_definition 1]
        }
        
        upvar $arg_name $arg_name
        set $arg_name $_value
    }
}



# ------------------------------------------------------------------
# ---------------------- Cookies -----------------------------------

proc cookies args {
    
    set command [lindex $args 0]
    set args [lrange $args 1 end]
    
    switch $command {
        
        clientvalue {
            # A string that is the concatenation of all cookies that
            # this http client wants to set. E.g. the value of the
            # "Cookie: " http header
            
            set result_list [list]
            foreach { name cookie } $::tclwebtest::cookies {
                catch { unset a_cookie }
                array set a_cookie $cookie
                lappend result_list "$name=$a_cookie(value)"
            }
            return [join $result_list "; "]
        }
        
        all {
            # return all cookies
            return $::tclwebtest::cookies
        }
        
        persistent {
            # return persistent cookies
            set result [list]
            foreach { name cookie } $::tclwebtest::cookies {
                catch { unset a_cookie }
                array set a_cookie $cookie
                if { [info exists a_cookie(persistent_p)] && 
                     $a_cookie(persistent_p) } {
                    lappend result $name $cookie
                }
            }
            return $result
        }
        
        set {
            # Add the given list of cookies. Typically used by a test
            # case to add persistent cookies, for example those of a
            # permanent login.
            
            # TODO maybe let this only be called at the beginning of a
            # session.
            
            # TODO only correctly deals with the output of "cookie
            # all" or "cookie persistent". Does not even throw an
            # error if input is not correct. It should check and set
            # the few needed values, so that cookies can be manually
            # written in test cases too.
            parse_args $args { } cookies_to_add

            if { [llength $cookies_to_add] == 0 } {
                log "'cookies set' was called with an empty list, so I assume we should clear the cookies"
                cookies clear
                return
            }

            array set a_cookies $::tclwebtest::cookies
            
            foreach { name cookie } $cookies_to_add {
                set a_cookies($name) $cookie
            }
            set ::tclwebtest::cookies [array get a_cookies]
        }

        clear {
            set ::tclwebtest::cookies [list]
        }

        default {
            error "the command \"cookies $command\" does not exist"
        }
    }
}


# Parses the value of the http "Set-cookie: " header into the global
# cookies list. Doesn't do anything with the path argument, e.g. if it
# is not / than this proc will behave wrongly.

# Also it currently depends on the Expires= information to be parsable
# by clock scan. If that is not the case then it will throw an error.
proc set_cookie { set_cookie_string } {
    
    array set a_cookies $::tclwebtest::cookies
    
    if ![regexp {([^=]+)=([^;]*)} $set_cookie_string match name value] {
        error "I did not understand this Cookie value: \"$set_cookie_string\". Please improve me."
    }
    
    set a_cookie(full) $set_cookie_string
    
    if { [regexp -nocase {expires\s*=\s*(.*)\s*$} $set_cookie_string match expires] } {
        set a_cookie(persistent_p) 1
        set a_cookie(expires) $expires
        
        set expires_seconds [clock scan $expires]
        
        if { $expires_seconds > [clock seconds] } {
            # A persistent cookie that wants to be set
            
            set a_cookie(persistent_p) 1
        } else {
            # A cookie that wants to be unset (either persistent or
            # not)
            
            catch { unset a_cookies($name) }
            set ::tclwebtest::cookies [array get a_cookies]
            return
            
        }
        #debug -lib "expires: $expires"
        
    } else {
        # A non-persistent cookie
        set a_cookie(persistent_p) 0
        set a_cookie(expires) ""
    }
    
    set a_cookie(name) $name ;# Note: the name is also stored in the outer cookie array
    set a_cookie(value) $value
    
    #debug -lib "string: $cookie_string"
    #debug -lib "name: $cookie_name, value: $cookie_value"
    
    # append this cookie
    set a_cookies($name) [array get a_cookie]
    set ::tclwebtest::cookies [array get a_cookies]
    
}

# --- end of Cookies -----------------------------------------------
# ------------------------------------------------------------------


# Save the given html result in a temporary file and launch a browser.
# Currently not satisfying since some referenced files such as images
# and stylesheets are required for a pleasing display. Maybe it is
# sufficient to parse the html for all those links, download the files
# and save them in a temporary directory, and replace them in the
# parsed html for local references.
proc open_browser { html { tmp_file "" } } {
    global tcl_platform
    
    if { $tmp_file == "" } {
        set tmp_file "/tmp/test_lib_tcl/tmp.html"
    }
    
    file mkdir [file dirname $tmp_file]
    
    set file [open $tmp_file w]
    #puts $file $html
    close $file
    
    switch $tcl_platform(platform) {
        "unix" {
            exec "netscape" "file://$tmp_file" "&"
        }
        "windows" {
            eval "exec [auto_execok start] [list "file://$tmp_file"] &"
        }
    }
}


# Do an http request and store everything in the current session

# Does not deal with URL's other than at the standard http port 80,
# because ::http:: does not include the port number in the Host:
# header and there seems no way to alter the Host: header manually. If
# it is set via the -headers option then there are two host headers,
# the wrong one and the one that was set explicitely. Grr. This has
# been entered at sf.net as Bug #452217.

# TODO do frames

# Expect a sane reply from the server, e.g. no status "500 Internal
# Server Error" or 404's and throws an assertion_failed
# otherwise. TODO not implemented. TODO define option to override
# this.
proc do_request args {
    link_variables regexp_defs
    parse_args $args { -followequiv -nocomplain } { url_or_link { query_key_values "" } }
    
    if $nocomplain {
        set nocomplain_option "-nocomplain"
    } else {
        set nocomplain_option ""
    }

    # As a global option. Added this here because i needed it after a
    # "form submit", but thats suboptimal.
    if { $::tclwebtest::FOLLOWEQUIV } {
        set followequiv 1
    }

    if $followequiv {
        set followequiv_option "-followequiv"
    } else {
        set followequiv_option ""
    }

    # reset all parts of the session that have to be reset
    set ::tclwebtest::links_extracted_p 0
    set ::tclwebtest::links [list]
    
    set ::tclwebtest::forms_extracted_p 0
    set ::tclwebtest::forms [list]
    set ::tclwebtest::current_form -1
    
    # TODO disabled. does anything use this?
    if { 0 && [llength $url_or_link] > 1 } {
        # this is an array list containing a link
        array set a_link $url_or_link
        set url $a_link(url)
    } else {
        set url $url_or_link
    }

    # remove any bookmark reference from the end of the url TODO write
    # a selftest
    regsub {#[^#]*$} $url {} url
    
    set url [absolute_link $url]
    set ::tclwebtest::url $url
    
    log "--- do_request for $url"

    # Kludge to deal with file:// urls. TODO implement this cleaner
    # into the do_request proc
    if { [string match "file://*" $url] } {
        return [do_request_file $url]
    }
    
    [http_namespace]::config -useragent $::tclwebtest::user_agent
    
    set headers [list]
    if { [llength $::tclwebtest::cookies] > 0 } {
        lappend headers "Cookie" [cookies clientvalue]
    }
    
    # Decide if we do GET or POST
    if { $query_key_values == "" } {
        # GET
        set token [[http_namespace]::geturl $url -headers $headers]
    } else {
        # POST
        set query_content [eval "[http_namespace]::formatQuery $query_key_values"]
        set token [[http_namespace]::geturl $url -query $query_content -headers $headers]
    }
    
    upvar #0 $token http_result
    
    regexp { (\d\d\d) } $http_result(http) full_match http_status
    #[string range $http_result(http) 0 2]
    log "http status: >>$http_status<<"
    set ::tclwebtest::http_status $http_status
    
    set meta $http_result(meta)
    
    # check if we received a Set-Cookie header, add to cookies if
    # necessary
    set meta_to_search $meta
    while { [regexp -nocase {set-cookie} $meta_to_search field_name] } {
        set lindex_of_cookie_header [lsearch -exact $meta_to_search $field_name]
        set cookie_string [lindex $meta_to_search [expr $lindex_of_cookie_header + 1]]
        set_cookie $cookie_string
        set meta_to_search [lrange $meta_to_search [expr $lindex_of_cookie_header + 2] end]
    }
    
    set ::tclwebtest::body $http_result(body)
    set ::tclwebtest::body_without_comments [strip_html_comments $http_result(body)]
    set ::tclwebtest::text [translate_entities [util_striphtml $::tclwebtest::body_without_comments]]
    
    
    # is it a redirect ?
    if { $http_status == "302" } {
        for { set i 0 } { $i < [llength $meta] } { incr i 2 } {
            if { [string match -nocase [lindex $meta $i] "location"] } {
                set location [lindex $meta [expr $i+1]]
                break
            }
        }
        log "following a redirect to: $location"
        eval "do_request $nocomplain_option $followequiv_option $location"
    } elseif { [regexp -expanded -nocase $regexp_script_before_html $http_result(body) match location] } {
        # a very silly form of redirect, with a <script> thingy before
        # the <html>
        log "stupid redirection to: $location"
        eval "do_request $nocomplain_option $followequiv_option $location"
    } elseif { $followequiv && [regexp -expanded -nocase $regexp_http_equiv_redirect $http_result(body) match delay location] } {
        # the http-equiv sort of redirect. ugh.
        log "stupid redir after $delay secondes to: $location"
        
        # optionally we could take the delay into account
        eval "do_request $nocomplain_option $followequiv_option $location"
        
    } elseif { $http_status != "200" } {
        debug -lib "$nocomplain"
        if { !$nocomplain } {
            assertion_failed "do_request did not return a page. HTTP status is $http_status"
        }
    }
    
    [http_namespace]::cleanup $token

    maybe_tidy
}

# open a file and load it. (Url expected to be file://)
proc do_request_file url {
    set path [string range $url [string length "file://"] end]
    
    set file [open $path]
    set ::tclwebtest::body [read $file]
    close $file
    
    set ::tclwebtest::body_without_comments [strip_html_comments $::tclwebtest::body]
    set ::tclwebtest::text [util_striphtml $::tclwebtest::body]
    
    # TODO think about what variables else have to be set here

    maybe_tidy
}

# Runs tidy on the current result page if the option TIDY is
# set. Output is sent to log_channel
proc maybe_tidy {} {
    variable TIDY
    variable log_channel "stdout"

    if { $TIDY } {
        # TODO check if tidy is available at all

        debug -lib "tidying"

        set ret_val "(nix)"
        global errorInfo errorCode
        set errorInfo ""
        log "--- TIDY START                  ---"
        if [catch {exec tidy -e -q << [response full] >&@ $log_channel}] {
            # Get the return value of the exec call
            if { [llength $errorCode] == 3 } {
                set ret_val [lindex $errorCode 2]
            } else {
                log "warning: there is some error in tclwebtests code, can't determine return value of tidy"
            }
        } else {
            # exec did not throw an error
            set ret_val 0
        }
        set url "URL: [response url]"
        switch $ret_val {
            0 { log "--- TIDY FINISH. NO COMPLAINTS. $url ---" }
            1 { log "--- TIDY FINISH. WARNINGS. $url ---" }
            2 { log "--- TIDY FINISH. ERRORS. $url ---" }
            default { log "--- TIDY FINISH. RETURNS: $ret_val. $url ---" }
        }
    }
}

proc response args {
    parse_args $args { } command
    
    switch $command {
        text { return $::tclwebtest::text }
        
        body { return $::tclwebtest::body }
        
        full { return $::tclwebtest::body }
        
        body_without_comments { return $::tclwebtest::body_without_comments }

        status { return $::tclwebtest::http_status }
        
        url { return $::tclwebtest::url }

        headers { error "TODO"; return $::tclwebtest::headers }
        
        default {
            error "The response subcommand \"response $command\" does not exist."
        }
    }
}


# Prepend host and context if either one is missing, derived from
# previously called url's

# TODO deal with this stuff: "../foo/bla"
proc absolute_link { url } {
    
    debug -lib "absolute_link called with url: $url, previous url was: $::tclwebtest::url"
    
    if { [string range $url 0 6] == "http://" || [string range $url 0 6] == "file://" || [string range $url 0 7] == "https://" } {
        return $url
    } else {
        if { $::tclwebtest::url == "" } {
            error "absolute_link was called with the relative link $url but no calls have been made before, so I don't know how to resolve it."
        }
        
        if { [string trim $url] == "" } {
            # We got a href="" -> same url again
            return $::tclwebtest::url
            
        } elseif { [string range $url 0 0] == "/" } {
            # append host
            regexp {(https?://[^/]+)} $::tclwebtest::url match host_part
            return "$host_part$url"
            
        } else {
            # append host and path
            
            # does not yet work for relative urls when only
            # http://bla.host.com has been called before. TODO
            regexp {([^?]*)/} $::tclwebtest::url match host_path_part

            if { $host_path_part == "http:/" } {
                # no idea if that fixes the TODO above but I need a
                # quick fix now
                set host_path_part $::tclwebtest::url
            }
            return "$host_path_part/$url"
        }
    }
}


# Extract all links from the body of this sessions request.
proc extract_links_if_necessary { } {
    link_variables regexp_defs
    
    if $::tclwebtest::links_extracted_p {
        return
    }

    debug -lib "extracting links"
    
    set ::tclwebtest::links [list]
    
    set html_to_search $::tclwebtest::body_without_comments
    
    while { [regexp -nocase -indices {<a[^>]*href[^>]*>([^<]|<[^/]|</[^a])*</a>} $html_to_search match_html match_url] } {
        
        
        catch { unset a_link }
        array set a_link [list]
        
        set a_link(start_idx)  [lindex $match_html 0]
        set a_link(stop_idx)  [lindex $match_html 1]
        
        set a_link(full) [string range $html_to_search $a_link(start_idx) $a_link(stop_idx)]
        set a_link(url) [get_attribute $a_link(full) href]
        
        # this is way too simple
        regexp -nocase {>(.*)<} $a_link(full) match a_link(content)
        set a_link(content) [normalize_html $a_link(content)]
        
        lappend ::tclwebtest::links [array get a_link]
        
        set html_to_search [string range $html_to_search [expr [lindex $match_html 1]+1] end]
    }
    
    
    set ::tclwebtest::current_link -1
    set ::tclwebtest::links_extracted_p 1
}


proc extract_forms_if_necessary { } {
    link_variables regexp_defs
    
    if $::tclwebtest::forms_extracted_p {
        return
    }
    
    # We misuse this var here by incrementing it for each new form, to
    # inform deal_with_field about the current form. After parsing we
    # set it to 0, or -1 if no forms are present.
    set ::tclwebtest::current_form -1
    set ::tclwebtest::current_field -1
    set ::tclwebtest::field_modified_p 0
    
    set ::tclwebtest::forms [list]
    
    set body $::tclwebtest::body_without_comments
    
    set start_idx 0
    
    # note: had a weird error when using: {<form[^>]*>.*?</form>}, for
    # whatever reason.
    while { [regexp -expanded -indices -nocase {<form[^>]*?>.*?</form>} [string range $body $start_idx end] form_match] } {
        
        catch { unset a_form }
        array set a_form [list]
        
        # get this form's full html blurb 
        set a_form(full) [string range $body \
                              [expr $start_idx + [lindex $form_match 0]] \
                              [expr $start_idx + [lindex $form_match 1]] \
                             ]
        
        incr ::tclwebtest::current_form
        
        regexp -nocase {<form[^>]*>} $a_form(full) form_tag 
        #puts "WOOOOOPS: found a form_tag: $form_tag"
        
        # get the action url
        set a_form(action) [get_attribute $form_tag action]
        
        # get the method, default is get
        set a_form(method) [get_attribute $form_tag method "get"]
        
        set a_form(name) [get_attribute $form_tag name]

        # get the content - everything that is inside the form but not
        # in html tags
        set a_form(content) [util_striphtml $a_form(full)]
        set a_form(fields) [list]
        
        # append the harvested form, deal_with_field does the rest
        lappend ::tclwebtest::forms [array get a_form]
        
        # get all fields of this form
        set start_field_idx 0
        
        while { [regexp -expanded -indices -nocase {<input[^>]*?>|<option[^>]*?>|<textarea[^>]*?>((?!</textarea>).)*</textarea>|<select[^>]*?>((?!</select>).)*</select>} [string range $a_form(full) $start_field_idx end] field_match] } {
            
            set abs_start [expr $start_idx + [lindex $form_match 0] + \
                               $start_field_idx + [lindex $field_match 0]]
            
            set abs_end [expr $start_idx + [lindex $form_match 0] + \
                             $start_field_idx + [lindex $field_match 1]]
            
            
            
            # yakk
            deal_with_field $abs_start $abs_end
            
            set start_field_idx [expr $start_field_idx + [lindex $field_match 1] + 1]
        }
        
        
        set start_idx [expr $start_idx + [lindex $form_match 1] + 1]
    }
    
    set ::tclwebtest::forms_extracted_p 1
    
    if { [llength $::tclwebtest::forms] == 0 } {
        set ::tclwebtest::current_form -1
        set ::tclwebtest::current_field -1
    } else {
        set ::tclwebtest::current_form 0
        set ::tclwebtest::current_field 0
    }
}


# Parse a field that is supposed to be on the specified position in
# the body. Add it to the fields list of the current_form (or modify
# the fields list if it is a radiobutton)
proc deal_with_field { start_idx stop_idx } {
    
    catch { unset a_result }
    array set a_result [list]
    
    set full [string range $::tclwebtest::body_without_comments $start_idx $stop_idx]
    
    set a_result(full) $full
    set a_result(start_idx) $start_idx
    set a_result(stop_idx) $stop_idx
    set append_p 1
    
    array set a_form [lindex $::tclwebtest::forms $::tclwebtest::current_form]
    set fields $a_form(fields)
    
    if { [regexp -nocase {<input} $full] } {
        # an input field
        set a_result(type) [string tolower [get_attribute $full type "text"]]
        set type $a_result(type)
        set a_result(name) [get_attribute $full name]
        set a_result(value) [get_attribute $full value]
        
        if { $type == "text" || $type == "hidden" || 
             $type == "password" } {
            
            set a_result(caption) [parse_caption_before $start_idx]
            
        } elseif { $type == "submit" } {
            
            
        } elseif { $type == "checkbox" } {
            
            # TODO: The regex below produces trouble when the word
            # "checked" occurs in some javascript inside the tag
            # (e.g. in an onClick thingy). Some cleaner html parsing
            # capabilities would do this program good ...
            if { [regexp -nocase {<input[^>]* checked[^>]*>} $full] } {
                set a_result(value) [get_attribute $full value]
            } else {
                set a_result(value) ""
            }

            set a_result(caption) [parse_caption_after $stop_idx]
            set a_result(choices) [list "" [get_attribute $full value]]
            
        } elseif { $type == "radio" } {
            
            # See if we have already a field of type radio
            # button with that name
            set name [get_attribute $full name]
            set value [get_attribute $a_result(full) value]           
            set caption [parse_caption_after $stop_idx]
            
            set found_idx [find_in_array_list -index $fields { ~n name } $name]
            
            if { $found_idx > -1 } {
                # This radiobutton is already here - add its value to
                # the existing choices.
                
                array set a_radio [lindex $fields $found_idx]
                lappend a_radio(choices) [list $value $caption]
                
                if { [regexp -nocase "<input.*checked.*>" $full] } {
                    set a_radio(value) $value
                }
                
                # replace the new field with the old one
                set fields [lreplace $fields $found_idx $found_idx [array get a_radio]]
                set a_form(fields) $fields
                set ::tclwebtest::forms [lreplace $::tclwebtest::forms $::tclwebtest::current_form $::tclwebtest::current_form [array get a_form]]
                
                set append_p 0
                
            } else {
                # First occurence of this radio button
                set a_result(caption) [parse_caption_before $start_idx]
                if { [regexp -nocase "<input.*checked.*>" $full] } {
                    set a_result(value) [get_attribute $a_result(full) value]
                } else {
                    set a_result(value) ""
                }
                lappend a_result(choices) [list [get_attribute $a_result(full) value] $caption]
            }
        } elseif { $type == "reset" } {
            log "Notice: there is no reason in the world to use a <input type=\"reset\"> field, yet this page does."
        } else {
            log "Notice: unimplemented input type: $type"
        }
        
    } elseif { [regexp -nocase -expanded {(<textarea[^>]*>)(.*?)</textarea>} $full bogus_match first_tag value] } {
        # a textarea
        set a_result(full) $full
        set a_result(type) textarea
        set a_result(name) [get_attribute $first_tag name]
        set a_result(value) $value
        set a_result(caption) [parse_caption_before $start_idx]
        
    } elseif { [regexp -nocase -expanded {(<select[^>]*>)(.*?)</select>} $full bogus_match first_tag body] } {
        # a select
        set a_result(full) $full
        set a_result(type) select
        set a_result(name) [get_attribute $first_tag name]
        set a_result(value) [list]
        set a_result(caption) [parse_caption_before $start_idx]
        
        # does this list allow multiple selections? TODO should not be
        # true if just the name attribute contains the word multiple
        set a_result(multiple_p) [expr [string first "multiple" $first_tag] > -1]
        
        # find all <option>... portions of the field, and stuff it
        # into the choices list. 

        # TODO at least mozilla seems to select the first option by
        # default, if the select is not multiple and no other option
        # is selected, this proc should do the same.
        set to_search $body
        set first_p 1
        while { [regexp -nocase {\s*(<option[^>]*?>.*?)\s*((<option[^>]*>|</option>|</select>).*?)?$} $to_search bogus_match full_option remaining] } {
            
            regexp -nocase {(<option[^>]*>)\s*(.*)$} $full_option bogus_match tag ch_caption
            
            
            # This is different wether value="" is given or nothing at
            # all. So there is this extremely_very ... kludge. TODO
            # should fix get_attribute to take a switch for that.
            set ch_value [get_attribute $tag value "extremely_veryunlikely_defaultname_01234567890333333"]
            debug -lib "SELECT ATT: $ch_value"
            if { $ch_value == "" } {
                set ch_value ""
            } elseif { $ch_value ==  "extremely_veryunlikely_defaultname_01234567890333333" } {
                # The value attribute is not set in the option tag, so
                # the value becomes the text that is included in the
                # tag body.
                set ch_value $ch_caption
            }
            
            lappend a_result(choices) [list $ch_value $ch_caption]
            
            # TODO this is not a perfect regexp
            if { [regexp -nocase "selected" $tag] } {
                if $a_result(multiple_p) {
                    lappend a_result(value) $ch_value
                } else {
                    set a_result(value) [list $ch_value]
                }
            }

            if { $first_p && !$a_result(multiple_p) } {
                # In single selects the first one is selected per
                # default
                set a_result(value) [list $ch_value]
                set first_p 0
            }
            
            set to_search $remaining
            set remaining ""
        }
        
    } else {
        error "cannot deal with this field: $full"
    }
    
    # TODO. to ensure all expected array fields are set
    if ![info exists a_result(caption)] {
        set a_result(caption) ""
    }
    
    # append this field to the current form
    if { $append_p } {
        set a_result(index) [llength $fields]

        set a_result(value) [translate_entities $a_result(value)]
        lappend fields [array get a_result]
        set a_form(fields) $fields
        set ::tclwebtest::forms [lreplace $::tclwebtest::forms $::tclwebtest::current_form $::tclwebtest::current_form [array get a_form]]
    }
    
} ;# end of deal_with_field



# Try to find a field's possible caption by scanning the text before
# it.
proc parse_caption_before position {
    
    set body $::tclwebtest::body_without_comments
    
    if { [string index $body $position] != "<" } {
        error "parse_caption_before called, but position ($position) does not point to a <, but to [string index $body $position]"
    }
    
    if { $position < 4 } {
        error "parse_caption_before called with a position of $position - i consider this insane and refuse further processing."
    }
    
    debug -lib "parse_caption_before $position"

    # Is this point inside <table></table> tags?
    
    # find the last <table> before position
    if { [set maybe_table_p [regexp -nocase -indices {.*(<table[^>]*>.*)<} [string range $body 0 $position] nada match]] } {
        set table_start [lindex $match 0]
        set table_first_part [string range $body $table_start $position]
        
        #debug -lib "t: $maybe_table_p at $table_start [string range $body $table_start $position]"

        # find the first </table> after position
        set offset $position
        if { [set maybe_table_p [regexp -nocase -indices {.*?(</table>)} [string range $body $offset end] nada match]] } {
            set table_stop [expr $offset + [lindex $match 1]]
            set table [string range $body $table_start $table_stop]
            
            if [regexp -nocase {<table[^>]*></?table[^>]*></table>} $table] {
                # There is at least one kind of table tag inside the found
                # range, which means the position is not inside a table or
                # there is a nested table inside. 
                
                # If there are nested tables then give up. This is by far not
                # the right way to test for this, but for now we assume that
                # in practice a nested table in a row means there won't be a
                # caption that we could find. (It's bad to use too many tables
                # anyway)
                debug -lib "giving up"
                return ""
            }
        }
    }
    
    # find the last <tr> before position
    if { [set maybe_table_p [regexp -nocase -indices {.*(<tr[^>]*>.*)<} [string range $body 0 $position] nada match]] } {
        set tr_start [lindex $match 0]
        set tr_first_part [string range $body $tr_start $position]
        
        # find the first </tr> after position
        set offset $position
        if [set maybe_table_p [regexp -nocase -indices {.*?(</tr>)} [string range $body $offset end] nada match]] {
            set tr_stop [expr $offset + [lindex $match 1]]
            set tr [string range $body $tr_start $tr_stop]
        }
    }
    
    #debug -lib "before final regex: $maybe_table_p, $tr_first_part"

    # Remove a bunch of tags that would be allowed inside a caption's
    # <td>. Easier then to dig again into the regex below (urgh).
    if { $maybe_table_p } {
        regsub -all -nocase "(</?b>|</?i>)" $tr_first_part "" tr_first_part
    }
    
    if { $maybe_table_p && [regexp -nocase {.*<(?:td|th)[^>]*>\s*(([[:alnum:]:_?!-]+)(\s+[[:alnum:]:_?!-]+)*)\s*</(?:td|th)>.*<} $tr_first_part nada caption] } {
        debug -lib "finally ..."
        return [normalize_html $caption]
    } else {
        # Not inside a table. Simply return everything from the left
        # of the <input> tag backwards till the first >.
        
        if [regexp {.*>\s*([^<]*?)\s*<} [string range $body 0 $position] nada match] {
            return [normalize_html $match]
        } else {
            return "not inside"
        }
    }
}

# Similar purpose to above, but simply parses till the first html element.
proc parse_caption_after position {
    
    if { [string index $::tclwebtest::body_without_comments $position] != ">" } {
        error "parse_caption_after called, but position ($position) does not point to a >, but to [string index $::tclwebtest::body_without_comments $position]"
    }
    
    set position [expr $position + 1]
    
    if ![regexp {\s*([^<]+?)\s*<} [string range $::tclwebtest::body_without_comments $position end] match caption] {
        return ""
    }
    return [normalize_html $caption]
}


# a wrapper around regexp that adds some functionality inspired by the
# limit function in the mutt mailclient: args can be more then one
# element and each will be interpreted as regexp against which string
# will be tested. Several lists with searchable content can be passed
# to this proc, and by default the first one will be searched. If an
# arg starts with ~ then the following character will identify which
# list to search. 

# TODO add example

# If an element of args is a single ! then the following regexp (with
# optional ~x) must not match.

# Always -nocase. Maybe it should be extended to be case sensitive
# when there is at least one upper case in the expression.

proc find_in_array_list args {
    
    parse_args $args { -index } { array_list modifier_key_list expression }
    array set a_keys $modifier_key_list
    
    # The first key is the default
    set default_key [lindex $modifier_key_list 1]
    
    set idx -1
    foreach item $array_list {
        incr idx
        
        catch { unset a_item }
        array set a_item $item
        
        set key_to_search $default_key
        set not ""
        foreach subex $expression {
            
            # is it NOT?
            if { $subex == "!" } {
                set not "!"
                continue
            }
            
            # is it a modifier?
            if { [regexp {~.} $subex] } {
                if { ![info exists a_keys($subex)] } {
                    error "\"$subex\" is not a valid modifier. I got this modifier_key_list: $modifier_key_list"
                }
                set key_to_search $a_keys($subex)
                continue
            }
            
            set found_p 1
            
   
            # it is an expression - test the element at the current
            # index
            set element $a_item($key_to_search)
            
            # don't use -expanded! Had a weird error when " " did not
            # match anymore (maybe I have to read the re_syntax man
            # pages) like [regexp -expanded {adresse andern}
            # {Lieferadresse andern} nada] returns 0. Hmm.
            if $not[regexp -nocase $subex $element] {
                # matches, on with the inner loop

                # but before that, we reset the active modifier to the
                # default
                set key_to_search $default_key
                continue
            } else {
                # does not match, stop the inner loop
                set found_p 0
                break
            }
            
            # reset the key for the next subex
            set key_to_search $default_key
        }
        
        if $found_p {
            if { $index } {
                return $idx
            } else {
                return $item
            }
        }
    }
    
    # looped through all list items but no match
    return ""
}


# split the given string into elements that are matched by the first
# parentheses pair in the regular expression and returns a list of all
# matches.
proc regsplit { rex string } {

    set result [list]

    while { [regexp -indices -nocase $rex $string bogus_match match] } {
        set first [lindex $match 0]
        set last [lindex $match 1]

        lappend result [string range $string $first $last]
        
        set string [string range $string $last end]
    }

    return $result
}


# there should be something like this that can be wrapped around some
# asserts like this:
#
# assert bla    # still good
# known_bug {
#   assert bli
#   assert blu
# }

# which would mean bli and blu are known bugs. This proc could then
# include the wrapped code in some reporting.

# It should also support aborting the whole test, so it can be put in the 
# beginning of a test file (makes sense when several test files are run
# at once)
proc known_bug { args } {
    parse_args $args { -abort } { args }

    if { $abort } {
        error "known bug" "known bug" { NONE }
    }
}



# Extracts the value of the named attribute out of the given html
# fragment. E.g. the url of href="..."
proc get_attribute { string att_name { default_value "" } } {
    
    if { [regexp -expanded -nocase "$att_name=\"(.*?)\"" $string match att_value] } {
        
    } elseif { [regexp -expanded -nocase "$att_name='(\[^>\]*?)'" $string match att_value] } {
    } elseif { [regexp -expanded -nocase "$att_name=(\[^ >\]*)(>| )" $string match att_value] } {
        
    } else {
        debug -lib "get_attribute: no attribute by that name: $att_name in $string"
        return $default_value
    }
    
    return $att_value
}


# loads a file an returns a few lines up to and including the
# linenumber
proc code_lines { filename linenumber } {
    set STACKTRACE_LENGTH 5

    set result ""

    set file [open $filename r]
    for { set i 1 } { $i <= $linenumber } { incr i } {
        if { ![gets $file line] == -1 } {
            break
        }

        if { [expr $linenumber - $i] < $STACKTRACE_LENGTH } {
            append result "$line\n"
        }
    }
    close $file
    return $result
}


# Yet another html-mangling proc. This one calls translate_entities
# and replaces all multiple occurences of whitespace with a single
# space. Thus it returns proper strings for text that e.g. spans
# several lines in the html source.
proc normalize_html html {
    set html [translate_entities $html]
    regsub -all {([[:space:]]+)} $html { } html
    return $html
}

# the following three procs are ripped from and therefore (C)
# arsdigita

# Returns a best-guess plain text version of an HTML fragment.  Better
# than ns_striphtml because it doesn't replace & g t ; and & l t ;
# with empty string.
proc util_striphtml {html} {
    return [util_expand_entities [util_remove_html_tags $html]]
}

proc util_remove_html_tags {html} {
    regsub -all {<[^>]*>} $html {} html

    # also remove all occurences of multiple linefeeds, just because
    # it looks better when debugging (e.g. by calling debug [response
    # text])
    regsub -all {\n(\s*)\n(\s*)\n+} $html "\n\n" html

    return $html
}


proc strip_html_comments { html } {
    regsub -all {<!--.*?-->} $html {} html
    return $html
}

proc util_expand_entities {html} {
    regsub -all {&lt;} $html {<} html
    regsub -all {&gt;} $html {>} html
    regsub -all {&quot;} $html {"} html
            regsub -all {&amp;} $html {\&} html
            return $html
        }


            # Grouped here because of the confusing effect of regexes on 
            # the font-locking of emacs.

            # TODO deal with value="Bla bla"
            variable regexp_form_action {<form[^>]+?action="?([^> "]+).*</form>}
            variable regexp_form_method {<form[^>]+?method="?([^> "]+).*</form>}
            variable regexp_input_name {<input[^>]+name="?([^" >]+)"?[^>]*>}
                                                           variable regexp_input_value {<input[^>]+value="?([^" >]+)"?[^>]+>}
            variable regexp_script_before_html {<script\s*>\s*document.location.href\s*=\s*"?'?([^"'< ]+)"?'?\s*</script>}
                                                                                                
                                                                                                # TODO should consider that this is inside a comment
                                                                                                variable regexp_http_equiv_redirect {<meta\s+http-equiv\s*=\s*"?refresh"?\s*content="?(\d)+;url=([^">]+)"?\s*>}

            set regexp_defs {
            variable regexp_form_action
            variable regexp_form_method
            variable regexp_input_name
            variable regexp_input_value
            variable regexp_script_before_html
            variable regexp_http_equiv_redirect
        }


# for emacs
if {1} {}
#"


# Source the given file (must be a filename, not a directory). Return
# 0 when successful, 1 when the test failed. Output is done via the
# log command, so it goes to wherever log_channel points to.
proc run_test { config_file file_name } {
    global errorInfo
    set start_time [clock seconds]

    ::tclwebtest::log "\n----- START: $file_name at \[[clock format $start_time -format "%d/%b/%Y:%H:%M:%S"]\] -----"
    reset_session

    if { [string length $config_file] > 0 } {
	set config_eval "source $config_file"
    } else {
	set config_eval ""
    }

    set to_eval "
      set TESTHOME [file dirname $file_name]
      $config_eval
      source $file_name
    "
    
    if [catch { eval $to_eval }] {

        if { [string first "known bug" $errorInfo] == 0 } {
            ::tclwebtest::log "known bug"
            # TODO needs something else the failed

        } else {
            # not a known bug

            if [regexp {(.*)\n--- end of assertionMsg ---\n} $errorInfo match assertionMsg] {
                ::tclwebtest::log "\n$assertionMsg"
            } else {
                # for whatever reason there is no assertionMsg, so just
                # append the stacktrace
                ::tclwebtest::log "\n$errorInfo"
            }
            
            # find the test file's line number in the stack trace
            if { [regexp "\\(file \"$file_name\" line (\\d+)\\)" $errorInfo match line] } {
                ::tclwebtest::log "in \"$file_name\" line $line:"
                ::tclwebtest::log [::tclwebtest::code_lines $file_name $line]
            }
        }
        
        ::tclwebtest::log "-----  FAILED: $file_name (took [expr [clock seconds] - $start_time]s)               -----\n"
        catch { flush $::tclwebtest::log_channel }
        return 1
        
    } else {
        ::tclwebtest::log "----- SUCCESS: $file_name (took [expr [clock seconds] - $start_time]s)               -----\n"
        catch { flush $::tclwebtest::log_channel }

        return 0
    }
}


} ;# end of namespace eval

